/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING 
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT 
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR 
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE 
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software 
* and to discontinue the availability of this software.  By using this software, 
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_serial_user.c
* Version      : 
* Device(s)    : R5F117GCG
* Tool-Chain   : IAR Systems iccrl78
* Description  : This file implements device driver for Serial module.
* Creation Date: 10/01/2015
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_serial.h"
/* Start user code for include. Do not edit comment generated here */
#define	UART_RX			RXD0
#define	UART_TX			TXD0
#define	UART_RX_STATUS		SSR01
#define	UART_TX_STATUS		SSR00
#define	UART_TX_INT_MASK	STMK0			// INTST0
#define	UART_RX_INT_MASK	SRMK0			// INTSR0
#define	UART_ERR_INT_MASK	SREMK0
#define UART_RE_INT		SIR01
#define	ASIF_TXBF		0x20			// tx buffer data flag
#define	SSR_TSF 		0x40			// tx communication status flag

#define TRUE			1
#define FALSE			0

#define		SK_UART_BUFSIZ	256			
#define		UART_BUF_MASK	(SK_UART_BUFSIZ -1)

/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
extern uint8_t * gp_uart0_tx_address;         /* uart0 send buffer address */
extern uint16_t  g_uart0_tx_count;            /* uart0 send data number */
extern uint8_t * gp_uart0_rx_address;         /* uart0 receive buffer address */
extern uint16_t  g_uart0_rx_count;            /* uart0 receive data number */
extern uint16_t  g_uart0_rx_length;           /* uart0 receive data length */
/* Start user code for global. Do not edit comment generated here */
uint8_t volatile gnSK_UART_RXst ;				
uint8_t volatile gnSK_UART_RXed ;			
uint8_t volatile gnSK_UART_TXst ;		
uint8_t volatile gnSK_UART_TXed ;	
uint8_t gaSK_UART_RXbuf[SK_UART_BUFSIZ];				
uint8_t gaSK_UART_TXbuf[SK_UART_BUFSIZ];
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: r_uart0_interrupt_receive
* Description  : This function is INTSR0 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
//#pragma vector = INTSR0_vect
__interrupt static void r_uart0_interrupt_receive(void)
{
    uint8_t rx_data;

    rx_data = RXD0;

    if (g_uart0_rx_length > g_uart0_rx_count)
    {
        *gp_uart0_rx_address = rx_data;
        gp_uart0_rx_address++;
        g_uart0_rx_count++;
    }
}

/***********************************************************************************************************************
* Function Name: r_uart0_interrupt_send
* Description  : This function is INTST0 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
//#pragma vector = INTST0_vect
__interrupt static void r_uart0_interrupt_send(void)
{
    if (g_uart0_tx_count > 0U)
    {
        TXD0 = *gp_uart0_tx_address;
        gp_uart0_tx_address++;
        g_uart0_tx_count--;
    }
}

/* Start user code for adding. Do not edit comment generated here */
#pragma vector = INTSRE0_vect
__interrupt static void UART_REInt(void) {
	volatile uint8_t		c;
	uint8_t	err_type;

	c = UART_RX;
	err_type = (uint8_t) (UART_RX_STATUS & 0x07U);
	UART_RE_INT = (uint16_t) err_type;
}
#pragma vector = INTSR0_vect
__interrupt static void UART_RxInt(void) {
	volatile uint8_t		ch;
	uint8_t			next, i;

	ch = UART_RX;
      i = gnSK_UART_RXed;
	next = (i + 1) & UART_BUF_MASK;
	if (gnSK_UART_RXst != next) {
		gaSK_UART_RXbuf[i] = ch;
		gnSK_UART_RXed = next;
	}
}

#pragma vector = INTST0_vect
__interrupt static void UART_TxInt(void) {
	uint8_t			i;
      i = gnSK_UART_TXed;
	if (gnSK_UART_TXst == i) {
		UART_TX_INT_MASK = TRUE;
		while((UART_TX_STATUS & (ASIF_TXBF))!=0) {};		
		return;
	}
	
	while((UART_TX_STATUS & (ASIF_TXBF))!=0) {
		NOP();
	};

	UART_TX = gaSK_UART_TXbuf[gnSK_UART_TXst];
	gnSK_UART_TXst = (gnSK_UART_TXst + 1)  & UART_BUF_MASK;
	
}



// -------------------------------------------------
//   Read from the serial port
// -------------------------------------------------
int16_t UART_GetChar(void) {
	uint8_t	ch, i;

      i = gnSK_UART_RXed;
	if (gnSK_UART_RXst == i) {
		return -1;
	} else {
		ch = gaSK_UART_RXbuf[gnSK_UART_RXst];
		gnSK_UART_RXst = (gnSK_UART_RXst + 1) & UART_BUF_MASK;
		return ch;
	}
}

// -------------------------------------------------
//   Write at the serial port
// -------------------------------------------------
void UART_PutChar(uint8_t ch) {
	uint8_t	next, i;
      i = gnSK_UART_TXed;
	if ((gnSK_UART_TXst == i) && ((UART_TX_STATUS & (ASIF_TXBF))==0)) {
		UART_TX = ch;
		//UART_TX_INT_MASK = TRUE;
		//while((UART_TX_STATUS & (ASIF_TXBF))!=0) {};
	} else {
		next = (gnSK_UART_TXed + 1) & UART_BUF_MASK;
		//if((UART_TX_STATUS & (ASIF_TXBF))==0) {
		//}
		if(gnSK_UART_TXst == next) {
			return;
		}
		gaSK_UART_TXbuf[gnSK_UART_TXed] = ch;
		gnSK_UART_TXed = next;
		UART_TX_INT_MASK = FALSE;
	}

}

uint16_t UART_GetLen(void) 
{
	uint16_t	len;
      uint8_t i;
      i = gnSK_UART_RXed;
	len = i - gnSK_UART_RXst;
	return len;
}

uint8_t UART_TxFinished(void) 
{
      uint8_t i;
      i = gnSK_UART_TXed;

  return (i - gnSK_UART_TXst == 0 && ((UART_TX_STATUS & SSR_TSF) == 0))?1:0;

} 
   

int putchar( int ch )
{
	UART_PutChar((uint8_t) ch);
	return ch;
}
/* End user code. Do not edit comment generated here */
