/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING 
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT 
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR 
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE 
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software 
* and to discontinue the availability of this software.  By using this software, 
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : user_main.c
* Version      : 
* Device(s)    : R5F117GCG
* Tool-Chain   : IAR Systems iccrl78
* Description  : This file implements main function.
* Creation Date: 10/01/2015
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_cgc.h"
#include "r_cg_port.h"
#include "r_cg_serial.h"
#include "r_cg_adc.h"
#include "r_cg_wdt.h"
#include "r_cg_it.h"
#include "r_cg_rtc.h"
#include "r_cg_timer.h"
#include "r_cg_pclbuz.h"
#include "r_cg_amp.h"
#include "r_cg_intc.h"
/* Start user code for include. Do not edit comment generated here */
#include <stdio.h>
#define BUZZER_ENABLE
#define CO_THRESHOLD    50
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
#define     MIN(x,y)    (x>y)?y:x
uint16_t    buzzer_timer;
uint8_t     buzzer_state;
uint8_t     smoke_state;
uint16_t    hush_count;
uint8_t     hush_mode       = 0;
uint16_t    smoke_count     = 0;
uint16_t    signal_average  = 0;
uint16_t    signal_min      = 0;
int8_t      threshold_count = 0;
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: check_smoke
* Description  : This function calculate smoke concentration.
* Arguments    : None
* Return Value : return 1 when smoke concentration larger than threshold, else return 0.
***********************************************************************************************************************/
uint8_t check_smoke(void)
{
        uint16_t s, min;

        s = adc_smoke - adc_bg;         // signal amplitude from IR pulse, eliminate background IR.
        min = (signal_min == 0) ?  (signal_average = signal_min = s) : MIN(s, signal_min);
        if(min + 30 > signal_min)     // ignore out of bound data.
        {   
            signal_min = (min + signal_min) / 2; 
        }
        if(s < signal_min + 30)        // ignore out of bound data.
        {   
            signal_average = (signal_average + s) / 2;
        }

        if(threshold_count != 0 && s < signal_average + 60)
        {
            threshold_count = 0;
        }
        else if(threshold_count == 0 && s > signal_average + 40)      // empirical value.
        {
            threshold_count++;
        }

        return (threshold_count >= 1) ? 1:0;
}
/***********************************************************************************************************************
* Function Name: user_main
* Description  : This function implements main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void user_main(void)
{
    //SENSOR_PWROFF = 0;
    // R_RTC_Start();
    R_IT10_Start();             // 3 seconds sampling rate.
    R_INTC0_Start(); //This function clears INTP0 interrupt flag and enables interrupt
    buzzer_state = BUZZER_IDLE;
    while (1U) {
        STOP();
        switch(gINTP0_int)
        {
            case INTC_FALLING:
                if(smoke_count == 0) 
                {
                    R_PCLBUZ1_Start();
                }
                else if(smoke_count > 1 && smoke_count < 20) // 60 seconds
                {
                    smoke_count = 0;
                    R_PCLBUZ1_Stop();
                    hush_mode = 1;
                    hush_count = 0;
                }
                gINTP0_int = INTC_IDLE;
                continue;
            case INTC_RISING:
                if(smoke_count == 0) 
                {
                    R_PCLBUZ1_Stop();
                }
                gINTP0_int = INTC_IDLE;
                continue;
            default:
                break;
        }
        if(hush_mode == 1)
        {
            hush_count++;
            if(hush_count > 20)
            {
                    hush_mode = 0;
            }
            continue;
        }
        switch(buzzer_state) 
        {
            case  BUZZER_ON:
                if(gSamplingTick - buzzer_timer > 1) 
                {   // Buzzer off for 6 seconds.
                    buzzer_timer = gSamplingTick;
                    buzzer_state = BUZZER_OFF;
                    R_PCLBUZ1_Stop();
                }
                break;
            case  BUZZER_OFF:
                if(gSamplingTick - buzzer_timer > 0) 
                {   // Buzzer off for 3 seconds.
                    buzzer_timer = gSamplingTick;
                    buzzer_state = BUZZER_IDLE;
                }
                else
                {
                    break;
                }
            case  BUZZER_IDLE:
                SENSOR_PWROFF = 0;        //Switch ON Power to sensors
                AMP_Start();              //switch ON Op Amps - Highspeed mode
                TRTCMP00  = 4;            //122us delay
                R_IT00_Start();           // Wait for op-amp to be stable.            
                while(adc_state != ADC_DONE) 
                {
                    HALT();
                }
                AMP_Stop();
                adc_state = ADC_IDLE;
                if(check_smoke() == 1)
                {
                    R_PCLBUZ1_Start();
                    buzzer_state = BUZZER_ON;
                    buzzer_timer = gSamplingTick;
                    smoke_count++;
                }
                else
                {
                    if(smoke_count > 0)
                    {
                        smoke_count--;
                    }
                }

                break;
        }
    }
}
